#!/usr/bin/env python2
#
# schlim_gui.py   --  Qt-based GUI code for schlim
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------
# 2010.09.19

fscreen = True          # It might be convenient to set this to False for testing
#fscreen = False

from PyQt4 import QtGui, QtCore
from collections import deque
import threading


class SchlimGui(QtGui.QApplication):
    def __init__(self):
        self.done = None
        self.eno = None
        QtGui.QApplication.__init__(self, [])
        self.setStyleSheet(theme["stylesheet"])
        # This overcomplicated looking bit should deal with translating the built in dialogs
        _translator = QtCore.QTranslator(self)
        if (_translator.load('qt_'+ QtCore.QLocale.system().name(),
                QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))):
            self.installTranslator(_translator)

        self.starting = True
        self.gview = QtGui.QGraphicsView()
        self.width = 0
        self.height = 0
        self.bgimage = None
        self.infobox = None
        self.messagebox = None
        self.scene = QtGui.QGraphicsScene()
        self.gview.setScene(self.scene)

        # Need some way of setting the widget sizes and positions from
        # the event loop, as the necessary information is not available
        # before this starts.
        #self.connect(self.scene,
        #        QtCore.SIGNAL("changed ( const QList<QRectF>& )"),
        #        self.newSize)
        self.connect(self, QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)"),
                self.newfocus)

        # callback list for event loop: (callback, arglist) pairs:
        self.idle_calls = deque()
        self.eno = QtCore.QEvent.registerEventType()

        if fscreen:
            self.gview.showFullScreen()

        img = theme['background_image']
        if img:
            self.setbgImage(img)

        self.fgimage = QtGui.QGraphicsPixmapItem()
        self.fgimage.setZValue(20)
        fgpm = QtGui.QPixmap(theme['loginsplash'])
        self.fgimage.setPixmap(fgpm)

        self.gview.activateWindow()


    def event(self, e):
        if e.type() == self.eno:
            # Process item from list
            cb, a = self.idle_calls.popleft()
            cb(*a)
            return True
        else:
            return QtGui.QApplication.event(self, e)


    def idle_add(self, callback, *args):
        self.idle_calls.append((callback, args))
        e = QtCore.QEvent(self.eno)
        self.postEvent(self, e)


    def start_gui(self):
        self.gview.show()
        return self.exec_()


    def makegui(self):
        if theme["showusers"]:
            ulw = self.scene.addWidget(self.userlistWidget())
            ulw.setGeometry(QtCore.QRectF(*theme["showusers_geometry"]))

        login_w = QtGui.QWidget()
        login__ = QtGui.QGridLayout()
        login_w.setLayout(login__)

        uw_l = QtGui.QLabel(_("Login name:"))
        self.username_w = QtGui.QLineEdit()
        self.connect(self.username_w, QtCore.SIGNAL("returnPressed()"),
                self.pwenter)
        # Add completer to user line edit
        completer = QtGui.QCompleter(self.userlist)
        completer.setCompletionMode(QtGui.QCompleter.InlineCompletion)
        self.username_w.setCompleter(completer)

        pw_l = QtGui.QLabel(_("Password:"))
        self.password_w = PwLineEdit(self)
        self.connect(self.password_w, QtCore.SIGNAL("returnPressed()"),
                self.handle_login)
        self.password_w.setEchoMode(QtGui.QLineEdit.Password)
        login_w.setLayout(login__)

        login__.addWidget(uw_l, 0, 0)
        login__.addWidget(self.username_w, 0, 1)
        login__.addWidget(pw_l, 1, 0)
        login__.addWidget(self.password_w, 1, 1)

        loginw = self.scene.addWidget(login_w)
        loginw.setGeometry(QtCore.QRectF(*theme["login_geometry"]))

        buttons_w = QtGui.QWidget()
        buttons__ = QtGui.QHBoxLayout()
        buttons_w.setLayout(buttons__)

        self.done = QtGui.QPushButton(_("Login"))
        #done.setAutoDefault(True)
        self.connect(self.done, QtCore.SIGNAL("clicked()"), self.handle_login)
        restart = QtGui.QPushButton(_("Restart"))
        self.connect(restart, QtCore.SIGNAL("clicked()"), self.handle_restart)
        shutdown = QtGui.QPushButton(_("Shutdown"))
        self.connect(shutdown, QtCore.SIGNAL("clicked()"), self.handle_shutdown)

        buttons__.addWidget(shutdown)
        buttons__.addWidget(restart)
        buttons__.addStretch()
        buttons__.addWidget(self.done)

        buttonsw = self.scene.addWidget(buttons_w)
        buttonsw.setZValue(10)
        buttonsw.setGeometry(QtCore.QRectF(*theme["buttons_geometry"]))


    def newSize(self):
        vp = self.gview.viewport()
        width = vp.width()
        height = vp.height()
        if self.width == width and self.height == height:
            return
        self.width = width
        self.height = height
        self.scene.setSceneRect(-width / 2.0, -height / 2.0, width, height)
        sw = float(self.width)
        sh = float(self.height)
        if self.bgimage:
            bgpm = self.bgimage.pixmap()
            w = bgpm.width()
            h = bgpm.height()

            iscale = theme.get('image_scale', '')
            if iscale.endswith('%'):
                sp = float(iscale.rstrip('%')) / 100
                ssw = sw * sp / w
                ssh = sh * sp / h
                s = ssw if ssw >= ssh else ssh
            elif iscale:
                bw, bh = [float(x) for x in iscale.split('_')]
                s = bw/w if bw/w < bh/h else bh/h
            else:
                s = 1.0
            sx = s
            sy = s
            t = QtGui.QTransform(sx, 0, 0, sy, -sx*w/2, -sy*h/2)
            self.bgimage.setTransform(t)

        fgpm = self.fgimage.pixmap()
        w = fgpm.width()
        h = fgpm.height()
        if theme['splash_fit']:
            s = sw/w if sw/w >= sh/h else sh/h
        else:
            s = 1.0
        sx = s
        sy = s
        t = QtGui.QTransform(sx, 0, 0, sy, -sx*w/2, -sy*h/2)
        self.fgimage.setTransform(t)


    def setbgColour(self, colour):
        self.scene.setBackgroundBrush(QtGui.QColor(colour))


    def setbgImage(self, path):
        """Set an image as canvas background.
        """
        if self.bgimage:
            if not path:
                self.scene.removeItem(self.bgimage)
                self.bgimage = None
                return
        else:
            self.bgimage = QtGui.QGraphicsPixmapItem()
            self.bgimage.setZValue(-100)
            self.scene.addItem(self.bgimage)
        bgpm = QtGui.QPixmap(path)
        self.bgimage.setPixmap(bgpm)


    def loginsplash(self, on=True):
        self.gview.setEnabled(not on)
        if on:
            self.scene.addItem(self.fgimage)
        else:
            self.scene.removeItem(self.fgimage)


    def info(self, message=None):
        if message:
            if not self.infobox:
                self.infobox = QtGui.QMessageBox()
            self.infobox.setText(message)
            self.infobox.setStandardButtons(self.infobox.NoButton)
            self.infobox.setIcon(self.infobox.Information)
            self.infobox.show()
            self.infobox.raise_()
        else:
            self.infobox.hide()


    def pwenter(self):
        user = self.username_w.text()
        self.password_w.setText("")
        self.password_w.setFocus()


    def newfocus(self, old, new):
        if new == self.gview:
            if self.starting:
                self.starting = False
                self.newSize()
                self.makegui()
            self.username_w.setFocus()


    def handle_login(self):
        u = unicode(self.username_w.text()).split('(', 1)[0].strip()
        p = unicode(self.password_w.text())
        self.loginsplash()
        self.runthread1(u, p)


    def login_done(self, em):
        if em == None:
            self.exit(0)
        elif em.startswith('USER='):
            em = self.login(em[5:])
            # ... which should only return if there was an error

        self.loginsplash(False)
        if em:
            self.message(em)
        self.password_w.setText("")
        self.password_w.setFocus()


    def message(self, text):
        if not self.messagebox:
            self.messagebox = QtGui.QMessageBox()
        self.messagebox.setText(text)
        self.messagebox.setIcon(self.messagebox.Warning)
        self.messagebox.show()
        self.messagebox.raise_()


    def handle_restart(self):
        self.runthread1("*restart*")


    def handle_shutdown(self):
        self.runthread1("*shutdown*")


    def runthread1(self, user, pw=None):
        threading.Thread(target=self._threadfunc, args=(user, pw)).start()


    def _threadfunc(self, user, pw):
        em = self.cb_login(user, pw)
        self.idle_add(self.login_done, em)


    def setUser(self, index):
        self.username_w.setText(self.userlist[index])
        self.username_w.setFocus()


    def userlistWidget(self):
        """Set up the list of users available for logging in.
        """
        uw = ListWidget([_("USERS")], selectionmode="Single")
        uw.set([[i] for i in self.userlist], -1)
        uw.connect("select", self.setUser)
        return uw


########### Qt widgets ############

class PwLineEdit(QtGui.QLineEdit):
    def __init__(self, ui):
        QtGui.QLineEdit.__init__(self)
        self.ui = ui

    def focusInEvent(self, e):
        user = unicode(self.ui.username_w.text())
        if user not in self.ui.userlist:
            self.ui.username_w.setFocus()
            return

        QtGui.QLineEdit.focusInEvent(self, e)


class ListWidget(QtGui.QTreeWidget):
    def __init__(self, headers, **kwargs):
        QtGui.QTreeWidget.__init__(self)
        self.setRootIsDecorated(False)

        self.mode = kwargs.get("selectionmode", "")
        if self.mode == "None":
            self.setSelectionMode(QtGui.QAbstractItemView.NoSelection)
        elif self.mode == "Single":
            self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        else:
            self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        self.setHeaderLabels(headers)
        hi = self.headerItem()

    def connect(self, signal, slot):
        if signal == "select":
            self._selectslot = slot
            QtCore.QObject.connect(self,
                    QtCore.SIGNAL("itemSelectionChanged()"),
                    self.s_select)

    def s_select(self):
        # Signal a selection change, passing the new selection list (indexes)
        s = [self.indexOfTopLevelItem(i) for i in self.selectedItems()] #qt
        if self.mode == "Single":
            s = s[0]
        self._selectslot(s)

    def set(self, items, index=0):
        # Note that each item must be a tuple/list containing
        # entries for each column.
        self.clear()
        c = 0
        for i in items:
            item = QtGui.QTreeWidgetItem(self, i)
            self.addTopLevelItem(item)
            if c == index:
                self.setCurrentItem(item)
            c += 1



class Thread1(QtCore.QThread):
    def process(self, cb, user, pw):
        self.user = user
        self.pw = pw
        self.cb = cb
        self.start()

    def run(self):
        em = self.cb(self.user, self.pw)
        self.emit(QtCore.SIGNAL("T1sig"), em)
