#!/usr/bin/env python2
#
# schlim_tidy.py   --  Tidy up remnants of old remote logins
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------
# 2010.10.21

import os, sys, pwd, time
from subprocess import call, Popen, PIPE, STDOUT

# Get configuration options
from schlim_conf import *
if os.path.isfile('/etc/schlim.conf'):
    execfile('/etc/schlim.conf')


def log(message):
    sys.stderr.write(message + '\n')


def clear_home():
    """The shares (remote home) directory should be empty.
    There should be no remote users in /etc/passwd.
    """
    # Remove any ICEAUTHORITY files
    call('rm -f /tmp/schlim_ICE_*', shell=True)

    # Clean up any volatile users
    for f in os.listdir('/tmp'):
        if f.startswith('schlim_home_'):
            with open('/etc/mtab') as fh:
                for line in fh:
                    ls = line.split()
                    if ls[0].endswith('/tmp/' + f):
                        # Need to kill all the user's processes before unmounting
                        call(['pkill', '-U', f.split('_', 2)[2]])
                        if call(['umount', ls[1]]) != 0:
                            log('***umount "%s" failed' % ls[1])
                            return False
                        break
            call(['rm', '-rf', '/tmp/' + f])

    # Check whether remote users are enabled
    if not HOMESDIR:
        return True

    # Remove any remote users
    for u in [p[0] for p in pwd.getpwall() if p[5].startswith(HOMESDIR)]:
        count = 0
        while True:
            call(['pkill', '-U', u])
            p = Popen(['userdel', u], stdout=PIPE, stderr=STDOUT)
            msg = p.communicate()[0]
            if p.returncode == 0:
                break
            count += 1
            if count == 20:
                log('***userdel %s:\n%s\n' % (u, msg))
                return False
            time.sleep(1)

    # Remove remote home directory mount
    for line in Popen('df', stdout=PIPE).communicate()[0].splitlines():
        if line.endswith(' ' + HOMESDIR):
            p = Popen(['fusermount', '-u', HOMESDIR], stdout=PIPE, stderr=STDOUT)
            msg = p.communicate()[0]
            if p.returncode != 0:
                log('***fusermount -u %s:\n%s\n' % (HOMESDIR, msg))
                return False

    return True


if __name__ == '__main__':
    sys.exit(0 if clear_home() else 1)
